"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const symbols_1 = require("./symbols");
const utils_1 = require("./utils");
const valit_1 = require("./valit");
const vality_1 = require("./vality");
vality_1.vality.array = (0, valit_1.valit)("array", e => (value, path, options) => {
    if (!Array.isArray(value))
        return { valid: false, errors: [{ message: "vality.array.base", path, options, value }] };
    const fn = (0, utils_1.enyToGuardFn)(e);
    const errors = [];
    for (const k in value) {
        // We can do this assertion here, since in the worst case, we'll get undefined, which is what we want to
        const res = fn(value[k], [...path, k]);
        errors.push(...res.errors);
        if (!res.valid && options.bail)
            break;
    }
    return { valid: errors.length === 0, errors };
}, {
    minLength: (val, o) => val.length >= o,
    maxLength: (val, o) => val.length <= o,
    bail: val => true,
});
vality_1.vality.object = (0, valit_1.valit)("object", e => (value, path, options) => {
    if (typeof value !== "object" || value === null)
        return { valid: false, errors: [{ message: "vality.object.base", path, options, value }] };
    const errors = [];
    // We iterate the passed object (in the model) first
    for (const k in e) {
        const ek = e[k];
        if (typeof ek === "object" && ek !== null && symbols_1._virtual in ek)
            continue; // We'll deal with these later
        // We can do this assertion here, since in the worst case, we'll get undefined, which is what we want to
        const res = (0, utils_1.enyToGuardFn)(ek)(value[k], [...path, k]);
        errors.push(...res.errors);
        if (!res.valid && options.bail)
            break;
    }
    // And then check for additioal keys
    for (const k in value) {
        const ek = e[k];
        if (ek === undefined || (typeof ek === "object" && ek !== null && symbols_1._virtual in ek)) {
            errors.push({
                message: "vality.object.extraProperty",
                path: [...path, k],
                options,
                value,
            });
            if (options.bail)
                break;
        }
    }
    return { valid: errors.length === 0, errors };
});
vality_1.vality.optional = (0, valit_1.valit)("optional", e => (val, path) => {
    if (val === undefined)
        return { valid: true, errors: [] };
    return (0, utils_1.enyToGuardFn)(e)(val, path);
});
vality_1.vality.enum = (0, valit_1.valit)("enum", (...es) => (value, path, options) => {
    const valid = es.some(e => (0, utils_1.enyToGuardFn)(e)(value, path).valid);
    return { valid, errors: valid ? [] : [{ message: "vality.enum.base", path, options, value }] };
});
vality_1.vality.tuple = (0, valit_1.valit)("tuple", (...es) => (value, path, options) => {
    if (!Array.isArray(value) || value.length !== es.length)
        return { valid: false, errors: [{ message: "vality.tuple.base", path, options, value }] };
    const errors = (0, utils_1.flat)(value.map((_, i) => i).map(i => (0, utils_1.enyToGuardFn)(es[i])(value[i], [...path, i]).errors));
    return { valid: errors.length === 0, errors };
});
// Gotta assert here as this is an exception where we don't just return your average valit, but need to add the _virtual marker
// This is required as vality.object checks for this symbol to correctly mark virtual proerties as "non-existend", not "required to be of value 'undefined'"
// Other valits could also mask this error in the future
// We still attach _validate, though, as, for whatever reason, this valit may still be called, and we really don't want a runtime error in that situation
vality_1.vality.virtual = () => ({
    [symbols_1._virtual]: true,
    [symbols_1._validate]: (value, path) => value === undefined
        ? { valid: true, errors: [] }
        : {
            valid: false,
            errors: [
                {
                    message: "vality.virtual.base",
                    path,
                    options: {},
                    value,
                },
            ],
        },
});
